//Von der Kamera die Daten holen

int KAMERA(void)
{

    char TX_String1[60], TX_String2[5], Cam_Data_Return[90], X_Data[60];

    /*
    In die Char-Arrays "String1"  und "String2" werden gleich zwei Strings gespeichert,
    die an die Kamera gesendet werden sollen.
    Im Char-Array "Cam_Data_Return" wird der gesammte String zwischengespeichert, der von der
    Kamera zurckgeliefert wird.
    Der komplette String, den uns die Kamera zurckliefern wird, besteht aus mehreren
    Einzelblcken. Ein solcher String kann z. B. die folgende Form haben:
    T 50 80 38 82 128 35 98
    Das "T" zeigt an, dass es sich um ein Datenpaket des Typs T handelt. Das
    Datenpaket "T" hat gem der Dokumentation der Kamera den folgenden Aufbau:
    "'T' - X_Data des Farbfleckmittelpunkts - Y-Wert des Farbfleckmittelpunkts - es
    folgen weitere Werte.
    Fr uns ist allein der zweite Block "X_Data des Farbfleckmittelpunkts"
    interessant.
    */

    int Ascii_Char, i, k, Camera_Data_Return, Ascii_1_Digit, Ascii_2_Digit, Ascii_3_Digit;

    /*
    Die Variablen AsciiErsteStelle, AsciiZweiteStelle und AsciiDritteStelle werden
    gleich im Rahmen einer Funktion
    zur Umwandlung von Dezimalzahlen verwendet.
    Die Zhl-Variablen i und k werden gleich beim Aufsplitten der Datenblcke
    bentigt.
    */

    TX_String1="tc 240 240 16 16 16 16";

    /*
    Der RGB-Wert, nach dem gesucht werden soll. Die Syntax lautet hier:
    tc - untere Grenze des R-Werts - obere Grenze des R-Werts - untere
    Grenze des G-Werts - obere Grenze des G-Werts usw.

    Alle Befehle mssen laut Kamera-Dokumenation mit einem \r beendet werden
    */

    TX_String2="\r";

    Serial_Init_IRQ(0,Buffer,80,60,SR_8BIT|SR_1STOP|SR_NO_PAR,SR_BD115200);

    /*
    Wir verwenden die Schnittstelle Nr. 1 (Das ist die zweite Schnittstelle,
    die Null mitgezhlt wird. Der Empfangspuffer betrgt 80 der Sendepuffer
    60. Es werden immer 8 Datenbits gesendet, gefolgt von einem Stopbit.
    Auf die Setzung eines Parittsbits wird verzichtet. Die Baudrate betrgt 115200.

    Die Funktion dieser Pause besteht darin, der Initialisierung
    der seriellen Schnittstelle etwas Zeit zu geben.
    */

    DELAY_MS(50);

    Serial_WriteText(0,TX_String1);
    Serial_WriteText(0,TX_String2);

    /*
    Hier noch einmal eine Pause, um der Kamera Zeit zu geben, die
    erhaltenen Infos zu verarbeiten.
    */

    DELAY_MS(50);

    /*
    Der Folgenden Anweisungsblock sorgt dafr, dass jeweils ein einzelenes
    Zeichen aus der seriellen Schnittstelle empfangen und in ein Array
    geschrieben werden. Die Funktion, die die Zeichen empfngt, lautet
    serial_ReadExt(). Sie ist in der C-Control-Bibliothek schon fertig
    definiert. Das Array, in das wie die vielen Einzelzeichen speichern,
    lautet "Cam_Data_Return".
    */


    i=0;
    do
    {
       Ascii_Char=Serial_ReadExt(0);
       Cam_Data_Return[i]=Ascii_Char;
       i++;
    }while(i<40);
    Cam_Data_Return[40]=0x30;

    /*
    Im den nchsten Anweisungsblcken geht es darum, das Array, in dem die
    vielen Einzelzeichen enthalten sind, in seine Bestandteile zu
    zerlegen. Dazu muss man wissen, in welcher Form die Daten der Kamera
    zurckgeliefert werden. Diese Form hngt davon ab, welchen Befehl
    wir an die Kamera geschickt haben. Wir haben vorhin einen "TC" (TraceColor)
    Befehl an die Kamera geschickt. Laut Dokumentation bekommt man in diesem
    Fall ein Datenpaket zurckgeliefert, das folgenden Aufbau hat:
    T<Leerzeichen>eine_bis_zu_dreistellige_Zahl<leerzeichen>wieder_eine_Zahl usw.
    Die erste Zahl, also die Zahl nach dem "T" und dem Leerzeichen ist
    fr uns unteressant, weil Sie die x-Koordinate des gefundenen
    Obejktes im virtuellen Bildausschnitt enthlt. Diese Zahl mssen
    wie extrahieren. Wir wissen ber die Zahl nicht genau, wie viele Stellen
    sie hat (zwischen 1 und 3). Wir wissen aber, dass vor ihr ein Leerzeichen
    steht und nach ihr ebenfalls. Das gilt in Prinzip fr alle einzelnen Werte,
    die in unserem Datenpaket enthalten sind. Das Datenpacket lsst sich in einzelene
    Blcke unterteilen, die durch Leerzeichen getrennt sind. Das Leerzeichen
    ist die entscheidende Information bei der Aufsplittung der Datenblcke.

    Wir extrahieren jetzt im folgenden Anweisungsblock den ersten Datenblock.
    Hierbei handelt es sich normalerweise um einen Block, der exakt so lautet:

    "T<Leerzeichen>"

    Das Datenpaket wird nicht mehr als 40 Zeichen enthalten
    */

    k=0;
    do
    {

        //"0x20" ist die Hexadezimalschreibweise fr ein Leerzeichen,
        //Taucht ein solches Lehrzeichen auf, kann die Schleife beendent werden
        if(Cam_Data_Return[k]==0x20)
        {
            k++;
            break;
        }
        k++;

    }while(k<40);

    /*
    Jetzt geschieht genau das, was mit dem ersten Datenblock geschah, mit
    dem nchsten Datenblock.

    Das zweite Paket wird ebenfalls niemals lnger als 40 Zeichen sein.
    */

    i=0;
    do
    {

        if(Cam_Data_Return[k]==0x20)
        {
            //"&H20" ist die Hexadezimalschreibweise fr ein Leerzeichen,
            //Taucht ein solches Lehrzeichen auf, kann die Schleife beendent werden
            i++;
            break;
        }

        X_Data[i]=Cam_Data_Return[k];
        k++;
        i++;

    }while(i<40);

    X_Data[i]=0x30;

    /*
    Die Schleife wurde verlassen. Das bedeutet, dass an den String "T " den
    wir extrahiert haben noch eine Terminierungsnull anghgen mssen. Die
    Terminierungnull muss immer an ein Chararkterarray angefgt werden.
    Andernfalls knnen whrend des Programmablaufs Veriablen berschrieben werden.

    Die Funktion der nchsten Anweisungsblcke besteht darin, die Zeichen, die
    wir von der seriellen Schnittstelle empfangen haben, in Integerzahlen umzuwandeln

    ber die serielle Schnittstelle knnen keine Zahlen, sondern nur Zeichen
    gesendet werden. Alle Zeichen erhalten nach dem ASCII-Zeichensatz eine Referenznummer
    Die Zahl 0 hat etwa die Referenznummer 48, die Zahl 1 die Referenznummer 49 usw.

    Um aus einer Referenznummer die Dezimalzahl zu gewinnen, muss man lediglich
    die Zahl 48 subtrahieren.

    Ein weiteres Problem besteht darin, dass wir noch nicht die Anzahl der Stellen kennen,
    die unsere Zahl hat. Sie kann ein- oder zweistellig sein. Verwenden wir eine hhere
    Bildschirmauflsung, so kann der Wert sogar dreistellig sein.
    Bei einer zweistelligen Zahl, muss die erste Ziffer mit 10 multipliziert werden,
    bevor sie mit der zweiten Ziffer addiert wird.
    Bei einer dreistelligen Zahl muss die erste Ziffer mit 100 und die zweite mit
    10 multipliziert werden.
    */

    //Die Zahl ist einstellig
    if(i==2)
    {
        Ascii_1_Digit=X_Data[0];
        Camera_Data_Return= Ascii_1_Digit-48;
    }

    //Die Zahl ist zweistellig
    if(i==3)
    {
        Ascii_1_Digit=X_Data[0];
        Ascii_2_Digit=X_Data[1];
        Camera_Data_Return=(Ascii_1_Digit-48)*10+Ascii_2_Digit-48;
    }

    //Die Zahl ist dreistellig
    if(i==4)
    {
        Ascii_1_Digit=X_Data[0];
        Ascii_2_Digit=X_Data[1];
        Ascii_3_Digit=X_Data[2];
        Camera_Data_Return=(Ascii_1_Digit-48)*100+(Ascii_2_Digit-48)*10+Ascii_3_Digit-48;
    }

    //Wert zurckliefern
    return Camera_Data_Return;

}



